Imports System
Imports System.Drawing
Imports System.Drawing.Color
Imports System.Reflection

' Color Plug-In class
' Dynamically loaded by the
' application.
Public Class ColorPlugIn
    Public Function ProcessImage(ByRef sourceImage As Bitmap, ByRef paramR As Integer, ByRef paramG As Integer, ByRef paramB As Integer)
        Dim maxPixValue As Integer
        Dim maxX As Integer
        Dim maxY As Integer
        Dim mycolor As Color
        Dim redValue, greenValue, blueValue As Integer

        Dim j As Integer
        Dim i As Integer
        ' Make sure color components are
        ' stored in 8 bits per pixel.
        ' If not throw an exception.
        Select Case sourceImage.PixelFormat
            Case Imaging.PixelFormat.Format24bppRgb, Imaging.PixelFormat.Format32bppArgb, Imaging.PixelFormat.Format32bppRgb, Imaging.PixelFormat.Format32bppPArgb
                maxPixValue = 255
            Case Else
                Throw (New Exception("Error, Pixel format not supported for this operation!"))
        End Select

        'Get the image dimensions
        maxX = sourceImage.Width
        maxY = sourceImage.Height

        ' Iterate through all Pixles in the image
        For i = 0 To (maxY - 1) Step 1
            For j = 0 To (maxX - 1) Step 1
                ' Get the Pixel value at the current Position.
                mycolor = sourceImage.GetPixel(j, i)
                ' Calculate the new color components
                ' of the current pixel.
                redValue = calcNewValue(mycolor.R, paramR, maxPixValue)
                greenValue = calcNewValue(mycolor.G, paramG, maxPixValue)
                blueValue = calcNewValue(mycolor.B, paramB, maxPixValue)
                ' Set the new color values.
                sourceImage.SetPixel(j, i, Color.FromArgb(redValue, greenValue, blueValue))
            Next
        Next
    End Function
    ' Helper method used to do the calculation of
    ' the color value and to check for overflow and
    ' underflow.
    Private Function calcNewValue(ByVal sourceValue As Integer, ByVal offset As Integer, ByVal maxPixValue As Integer) As Integer
        ' Add the trackBar value of color 
        ' to the current color value.
        Dim result As Integer = sourceValue + offset
        ' Check for under and overflow.
        If (result > maxPixValue) Then
            Return maxPixValue
        ElseIf (result < 0) Then
            Return 0
        Else
            Return result
        End If
    End Function


End Class
