using System;
using System.Drawing;

namespace Photo_Editor_Application
{
    /// <summary>
    /// The CircleComponent class implements the
    /// functionality to draw a circle into the picture.
    /// </summary>
    /// <requirement>F:image_graphics_annotations</requirement>
    public class CircleComponent : GraphicsComponent
    {
        /// <summary>
        /// Constructor for circle component.
        /// </summary>
        public CircleComponent()
        {

        }

        /// <summary>
        /// Draws the circle in the provided Graphics object.
        /// Calculates the bouding box so width and height are 
        /// allways > 0.
        /// </summary>
        public override void Draw()
        {
            int width =  CalcViewportCoords(EndPoint).X - CalcViewportCoords(StartPoint).X;
            int height = CalcViewportCoords(EndPoint).Y - CalcViewportCoords(StartPoint).Y;

            if(width < 0 && height < 0)
                DeviceContext.DrawEllipse(DrawingPen, CalcViewportCoords(EndPoint).X, CalcViewportCoords(EndPoint).Y, 
                    Math.Abs(width), Math.Abs(height));
            else if (width < 0)
                DeviceContext.DrawEllipse(DrawingPen, CalcViewportCoords(EndPoint).X, CalcViewportCoords(StartPoint).Y, 
                    Math.Abs(width), height);
            else if (height < 0)
                DeviceContext.DrawEllipse(DrawingPen, CalcViewportCoords(StartPoint).X, CalcViewportCoords(EndPoint).Y, 
                    width, Math.Abs(height));
            else 
                DeviceContext.DrawEllipse(DrawingPen, CalcViewportCoords(StartPoint).X, CalcViewportCoords(StartPoint).Y, 
                    width, height);
        }
    }

}
