using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Microsoft.ApplicationBlocks.ExceptionManagement;
using System.IO;

namespace Photo_Editor_Application
{
    /// <summary>
    /// Summary description for Form1.
    /// </summary>
    public class PhotoEditorForm : System.Windows.Forms.Form
    {
        private System.Windows.Forms.MainMenu MainMenu;
        private System.Windows.Forms.MenuItem FileMenu;
        private System.Windows.Forms.MenuItem OpenFile;
        
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;
        /// <summary>
        /// Holds a custom scrollable control object.
        /// </summary>
        public CustomScrollableControl.CustomScrollableControl customScrollableControl;
        private System.Windows.Forms.TabControl tabControl;
        private System.Windows.Forms.TabPage basicImageOperations;
        private System.Windows.Forms.Button cropImageButton;
        private System.Windows.Forms.Button flipVerticalButton;
        private System.Windows.Forms.Button flipHorizontalButton;
        private System.Windows.Forms.Button rotateClockButton;
        private System.Windows.Forms.Button rotateCClockButton;
        private System.Windows.Forms.MenuItem saveMenu;
        private System.Windows.Forms.TabPage Overlays;
        /// <summary>
        /// Holds a Picture Object.
        /// </summary>
        public Picture PictureObject;
        private System.Windows.Forms.GroupBox groupBox1;
        private System.Windows.Forms.Button RectangleButton;
        private System.Windows.Forms.Button CircleButton;
        private System.Windows.Forms.Button ROIButton;
        private System.Windows.Forms.Button EraseButton;
        private System.Windows.Forms.Button LineButton;
        private System.Windows.Forms.Button TextButton;
        private System.Windows.Forms.Button ApplyButton;
        private System.Windows.Forms.Button ResetButton;
        private System.Windows.Forms.MenuItem menuItem1;
        private System.Windows.Forms.MenuItem menuItem2;
        private System.Windows.Forms.NumericUpDown penNumericUpDown;
        private System.Windows.Forms.GroupBox lineThickness;
        private System.Windows.Forms.Button selectColorButton;
        private System.Windows.Forms.RadioButton EllipseFrameRadioButton;
        private System.Windows.Forms.RadioButton RectFrameRadioButton;
        private System.Windows.Forms.GroupBox groupBox2;
        private System.Windows.Forms.TabPage tabPage1;
        private System.Windows.Forms.PictureBox picturePreView;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.GroupBox groupBox3;
        private System.Windows.Forms.TrackBar trackBarContrast;
        private System.Windows.Forms.Button ApplyProcessingButton;
        private System.Windows.Forms.Button IPResetButton;
        private System.Windows.Forms.GroupBox Brightness;
        private System.Windows.Forms.TrackBar trackBarBrightness;
        private System.Windows.Forms.GroupBox groupBox4;
        private System.Windows.Forms.TrackBar trackBarGreen;
        private System.Windows.Forms.TrackBar trackBarBlue;
        private System.Windows.Forms.TrackBar trackBarRed;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.Label label4;
        private System.Windows.Forms.Button redEyeButton;
		private System.Windows.Forms.TabPage tabPage2;
		private System.Windows.Forms.Button fontButton;
		private System.Windows.Forms.FontDialog fontDialog;
		private System.Windows.Forms.Button colorButton;
		private System.Windows.Forms.NumericUpDown rotationX;
		private System.Windows.Forms.NumericUpDown rotationY;
		private System.Windows.Forms.NumericUpDown rotationZ;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.GroupBox groupBox5;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Button textApplyButton;
		private System.Windows.Forms.Button textUndoButton;
		private System.Windows.Forms.ColorDialog colorDialog;
		private System.Windows.Forms.TextBox textBox;
		private System.Windows.Forms.Button placementButton;

        // Get the directory in which the application was started.
        private static string applicationDirectory = Application.StartupPath;
        
        /// <summary>
        /// Accessor to the Application directory.
        /// </summary>
        public static string GetApplicationDirectory
        {
            get
            {
                return applicationDirectory;
            }
        }
        private TextBoxInput inputTextBox;
        /// <summary>
        /// Constructor.
        /// </summary>
        public PhotoEditorForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //
            
            PictureObject = new Picture();
            //sets the private property to not drawing
            isDrawing = false;

            // Sets the static property of the Component to the
            // Graphics object of the window it is drawn in
            GraphicsComponent.DeviceContext = customScrollableControl.CreateGraphics();  
            // Create TextBoxInput to get the input text
            inputTextBox = new TextBoxInput();
            // Create an instance of the plug in interface class
            // in order to be able to use dynamic loading.
            plugInInterface = new PlugInInterface();

            // Create a thumbnail image from the loaded image.
            // The image will be displayed in the preview of the 
            // Image processing tab.
            thumbImage = PictureObject.LoadedImage.GetThumbnailImage( 
                picturePreView.Width, 
                picturePreView.Height,
                null, 
                IntPtr.Zero);

            picturePreView.Image = thumbImage;

        }

        private PlugInInterface plugInInterface;
        /// <summary>
        /// Custom paint method.
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            try
            {
                Graphics deviceContext = e.Graphics;
          
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        } 

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if (components != null) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(PhotoEditorForm));
            this.MainMenu = new System.Windows.Forms.MainMenu();
            this.FileMenu = new System.Windows.Forms.MenuItem();
            this.OpenFile = new System.Windows.Forms.MenuItem();
            this.saveMenu = new System.Windows.Forms.MenuItem();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.customScrollableControl = new CustomScrollableControl.CustomScrollableControl();
            this.tabControl = new System.Windows.Forms.TabControl();
            this.basicImageOperations = new System.Windows.Forms.TabPage();
            this.rotateCClockButton = new System.Windows.Forms.Button();
            this.flipHorizontalButton = new System.Windows.Forms.Button();
            this.rotateClockButton = new System.Windows.Forms.Button();
            this.flipVerticalButton = new System.Windows.Forms.Button();
            this.cropImageButton = new System.Windows.Forms.Button();
            this.Overlays = new System.Windows.Forms.TabPage();
            this.selectColorButton = new System.Windows.Forms.Button();
            this.penNumericUpDown = new System.Windows.Forms.NumericUpDown();
            this.ResetButton = new System.Windows.Forms.Button();
            this.ApplyButton = new System.Windows.Forms.Button();
            this.TextButton = new System.Windows.Forms.Button();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.RectangleButton = new System.Windows.Forms.Button();
            this.CircleButton = new System.Windows.Forms.Button();
            this.EraseButton = new System.Windows.Forms.Button();
            this.ROIButton = new System.Windows.Forms.Button();
            this.LineButton = new System.Windows.Forms.Button();
            this.lineThickness = new System.Windows.Forms.GroupBox();
            this.groupBox2 = new System.Windows.Forms.GroupBox();
            this.RectFrameRadioButton = new System.Windows.Forms.RadioButton();
            this.EllipseFrameRadioButton = new System.Windows.Forms.RadioButton();
            this.tabPage1 = new System.Windows.Forms.TabPage();
            this.redEyeButton = new System.Windows.Forms.Button();
            this.groupBox4 = new System.Windows.Forms.GroupBox();
            this.label4 = new System.Windows.Forms.Label();
            this.label3 = new System.Windows.Forms.Label();
            this.label2 = new System.Windows.Forms.Label();
            this.trackBarRed = new System.Windows.Forms.TrackBar();
            this.trackBarBlue = new System.Windows.Forms.TrackBar();
            this.trackBarGreen = new System.Windows.Forms.TrackBar();
            this.Brightness = new System.Windows.Forms.GroupBox();
            this.trackBarBrightness = new System.Windows.Forms.TrackBar();
            this.IPResetButton = new System.Windows.Forms.Button();
            this.ApplyProcessingButton = new System.Windows.Forms.Button();
            this.groupBox3 = new System.Windows.Forms.GroupBox();
            this.trackBarContrast = new System.Windows.Forms.TrackBar();
            this.label1 = new System.Windows.Forms.Label();
            this.picturePreView = new System.Windows.Forms.PictureBox();
            this.tabPage2 = new System.Windows.Forms.TabPage();
            this.placementButton = new System.Windows.Forms.Button();
            this.textUndoButton = new System.Windows.Forms.Button();
            this.textApplyButton = new System.Windows.Forms.Button();
            this.label8 = new System.Windows.Forms.Label();
            this.label5 = new System.Windows.Forms.Label();
            this.rotationX = new System.Windows.Forms.NumericUpDown();
            this.colorButton = new System.Windows.Forms.Button();
            this.fontButton = new System.Windows.Forms.Button();
            this.textBox = new System.Windows.Forms.TextBox();
            this.rotationY = new System.Windows.Forms.NumericUpDown();
            this.rotationZ = new System.Windows.Forms.NumericUpDown();
            this.label6 = new System.Windows.Forms.Label();
            this.label7 = new System.Windows.Forms.Label();
            this.groupBox5 = new System.Windows.Forms.GroupBox();
            this.fontDialog = new System.Windows.Forms.FontDialog();
            this.colorDialog = new System.Windows.Forms.ColorDialog();
            this.tabControl.SuspendLayout();
            this.basicImageOperations.SuspendLayout();
            this.Overlays.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.penNumericUpDown)).BeginInit();
            this.groupBox1.SuspendLayout();
            this.groupBox2.SuspendLayout();
            this.tabPage1.SuspendLayout();
            this.groupBox4.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarRed)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarBlue)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarGreen)).BeginInit();
            this.Brightness.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarBrightness)).BeginInit();
            this.groupBox3.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarContrast)).BeginInit();
            this.tabPage2.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.rotationX)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.rotationY)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.rotationZ)).BeginInit();
            this.SuspendLayout();
            // 
            // MainMenu
            // 
            this.MainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                     this.FileMenu});
            // 
            // FileMenu
            // 
            this.FileMenu.Index = 0;
            this.FileMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                     this.OpenFile,
                                                                                     this.saveMenu,
                                                                                     this.menuItem1,
                                                                                     this.menuItem2});
            this.FileMenu.Text = "&File";
            // 
            // OpenFile
            // 
            this.OpenFile.Index = 0;
            this.OpenFile.Text = "&Open File";
            this.OpenFile.Click += new System.EventHandler(this.OpenFile_Click);
            // 
            // saveMenu
            // 
            this.saveMenu.Index = 1;
            this.saveMenu.Text = "&Save as ...";
            this.saveMenu.Click += new System.EventHandler(this.saveMenu_Click);
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 2;
            this.menuItem1.Text = "-";
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 3;
            this.menuItem2.Text = "Exit";
            this.menuItem2.Click += new System.EventHandler(this.menuItem2_Click);
            // 
            // customScrollableControl
            // 
            this.customScrollableControl.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
                | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right)));
            this.customScrollableControl.Location = new System.Drawing.Point(0, 0);
            this.customScrollableControl.Name = "customScrollableControl";
            this.customScrollableControl.ScrollingImage = null;
            this.customScrollableControl.ScrollingImageArea = new System.Drawing.Rectangle(0, 0, 826, 447);
            this.customScrollableControl.Size = new System.Drawing.Size(843, 464);
            this.customScrollableControl.TabIndex = 0;
            this.customScrollableControl.Load += new System.EventHandler(this.customScrollableControl_Load);
            this.customScrollableControl.MouseUp += new System.Windows.Forms.MouseEventHandler(this.customScrollableControl_MouseUp);
            this.customScrollableControl.Paint += new System.Windows.Forms.PaintEventHandler(this.customScrollableControl_Paint);
            this.customScrollableControl.MouseEnter += new System.EventHandler(this.customScrollableControl_MouseEnter);
            this.customScrollableControl.MouseMove += new System.Windows.Forms.MouseEventHandler(this.customScrollableControl_MouseMove);
            this.customScrollableControl.MouseLeave += new System.EventHandler(this.customScrollableControl_MouseLeave);
            this.customScrollableControl.MouseDown += new System.Windows.Forms.MouseEventHandler(this.customScrollableControl_MouseDown);
            // 
            // tabControl
            // 
            this.tabControl.Controls.Add(this.basicImageOperations);
            this.tabControl.Controls.Add(this.Overlays);
            this.tabControl.Controls.Add(this.tabPage1);
            this.tabControl.Controls.Add(this.tabPage2);
            this.tabControl.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.tabControl.Location = new System.Drawing.Point(0, 467);
            this.tabControl.Name = "tabControl";
            this.tabControl.SelectedIndex = 0;
            this.tabControl.Size = new System.Drawing.Size(843, 216);
            this.tabControl.TabIndex = 1;
            // 
            // basicImageOperations
            // 
            this.basicImageOperations.Controls.Add(this.rotateCClockButton);
            this.basicImageOperations.Controls.Add(this.flipHorizontalButton);
            this.basicImageOperations.Controls.Add(this.rotateClockButton);
            this.basicImageOperations.Controls.Add(this.flipVerticalButton);
            this.basicImageOperations.Controls.Add(this.cropImageButton);
            this.basicImageOperations.Location = new System.Drawing.Point(4, 25);
            this.basicImageOperations.Name = "basicImageOperations";
            this.basicImageOperations.Size = new System.Drawing.Size(835, 187);
            this.basicImageOperations.TabIndex = 0;
            this.basicImageOperations.Text = "Basic Image Operations";
            // 
            // rotateCClockButton
            // 
            this.rotateCClockButton.Image = ((System.Drawing.Image)(resources.GetObject("rotateCClockButton.Image")));
            this.rotateCClockButton.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.rotateCClockButton.Location = new System.Drawing.Point(307, 102);
            this.rotateCClockButton.Name = "rotateCClockButton";
            this.rotateCClockButton.Size = new System.Drawing.Size(163, 26);
            this.rotateCClockButton.TabIndex = 4;
            this.rotateCClockButton.Text = "Rotate CClockwise";
            this.rotateCClockButton.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.rotateCClockButton.Click += new System.EventHandler(this.rotateCClockButton_Click);
            // 
            // flipHorizontalButton
            // 
            this.flipHorizontalButton.Image = ((System.Drawing.Image)(resources.GetObject("flipHorizontalButton.Image")));
            this.flipHorizontalButton.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.flipHorizontalButton.Location = new System.Drawing.Point(154, 102);
            this.flipHorizontalButton.Name = "flipHorizontalButton";
            this.flipHorizontalButton.Size = new System.Drawing.Size(124, 26);
            this.flipHorizontalButton.TabIndex = 3;
            this.flipHorizontalButton.Text = "Flip Horizontal";
            this.flipHorizontalButton.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.flipHorizontalButton.Click += new System.EventHandler(this.flipHorizontalButton_Click);
            // 
            // rotateClockButton
            // 
            this.rotateClockButton.Image = ((System.Drawing.Image)(resources.GetObject("rotateClockButton.Image")));
            this.rotateClockButton.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.rotateClockButton.Location = new System.Drawing.Point(307, 55);
            this.rotateClockButton.Name = "rotateClockButton";
            this.rotateClockButton.Size = new System.Drawing.Size(163, 27);
            this.rotateClockButton.TabIndex = 2;
            this.rotateClockButton.Text = "Rotate Clockwise";
            this.rotateClockButton.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.rotateClockButton.Click += new System.EventHandler(this.rotateClockButton_Click);
            // 
            // flipVerticalButton
            // 
            this.flipVerticalButton.Image = ((System.Drawing.Image)(resources.GetObject("flipVerticalButton.Image")));
            this.flipVerticalButton.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.flipVerticalButton.Location = new System.Drawing.Point(154, 55);
            this.flipVerticalButton.Name = "flipVerticalButton";
            this.flipVerticalButton.Size = new System.Drawing.Size(124, 27);
            this.flipVerticalButton.TabIndex = 1;
            this.flipVerticalButton.Text = "Flip Vertical";
            this.flipVerticalButton.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.flipVerticalButton.Click += new System.EventHandler(this.flipVerticalButton_Click);
            // 
            // cropImageButton
            // 
            this.cropImageButton.Image = ((System.Drawing.Image)(resources.GetObject("cropImageButton.Image")));
            this.cropImageButton.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
            this.cropImageButton.Location = new System.Drawing.Point(19, 55);
            this.cropImageButton.Name = "cropImageButton";
            this.cropImageButton.Size = new System.Drawing.Size(115, 27);
            this.cropImageButton.TabIndex = 0;
            this.cropImageButton.Text = "Crop Image";
            this.cropImageButton.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
            this.cropImageButton.Click += new System.EventHandler(this.cropImageButton_Click);
            // 
            // Overlays
            // 
            this.Overlays.Controls.Add(this.selectColorButton);
            this.Overlays.Controls.Add(this.penNumericUpDown);
            this.Overlays.Controls.Add(this.ResetButton);
            this.Overlays.Controls.Add(this.ApplyButton);
            this.Overlays.Controls.Add(this.TextButton);
            this.Overlays.Controls.Add(this.groupBox1);
            this.Overlays.Controls.Add(this.lineThickness);
            this.Overlays.Controls.Add(this.groupBox2);
            this.Overlays.Location = new System.Drawing.Point(4, 25);
            this.Overlays.Name = "Overlays";
            this.Overlays.Size = new System.Drawing.Size(835, 187);
            this.Overlays.TabIndex = 1;
            this.Overlays.Text = "Graphics and Text";
            // 
            // selectColorButton
            // 
            this.selectColorButton.Location = new System.Drawing.Point(584, 88);
            this.selectColorButton.Name = "selectColorButton";
            this.selectColorButton.Size = new System.Drawing.Size(88, 23);
            this.selectColorButton.TabIndex = 15;
            this.selectColorButton.Text = "Select Color";
            this.selectColorButton.Click += new System.EventHandler(this.selectColorButton_Click);
            // 
            // penNumericUpDown
            // 
            this.penNumericUpDown.Location = new System.Drawing.Point(600, 48);
            this.penNumericUpDown.Maximum = new System.Decimal(new int[] {
                                                                             20,
                                                                             0,
                                                                             0,
                                                                             0});
            this.penNumericUpDown.Minimum = new System.Decimal(new int[] {
                                                                             1,
                                                                             0,
                                                                             0,
                                                                             0});
            this.penNumericUpDown.Name = "penNumericUpDown";
            this.penNumericUpDown.Size = new System.Drawing.Size(64, 22);
            this.penNumericUpDown.TabIndex = 13;
            this.penNumericUpDown.Value = new System.Decimal(new int[] {
                                                                           1,
                                                                           0,
                                                                           0,
                                                                           0});
            this.penNumericUpDown.ValueChanged += new System.EventHandler(this.penNumericUpDown_ValueChanged);
            // 
            // ResetButton
            // 
            this.ResetButton.Location = new System.Drawing.Point(712, 88);
            this.ResetButton.Name = "ResetButton";
            this.ResetButton.Size = new System.Drawing.Size(90, 26);
            this.ResetButton.TabIndex = 12;
            this.ResetButton.Text = "Reset";
            this.ResetButton.Click += new System.EventHandler(this.ResetButton_Click);
            // 
            // ApplyButton
            // 
            this.ApplyButton.Location = new System.Drawing.Point(712, 48);
            this.ApplyButton.Name = "ApplyButton";
            this.ApplyButton.Size = new System.Drawing.Size(90, 27);
            this.ApplyButton.TabIndex = 11;
            this.ApplyButton.Text = "Apply";
            this.ApplyButton.Click += new System.EventHandler(this.ApplyButton_Click);
            // 
            // TextButton
            // 
            this.TextButton.Location = new System.Drawing.Point(584, 120);
            this.TextButton.Name = "TextButton";
            this.TextButton.Size = new System.Drawing.Size(90, 27);
            this.TextButton.TabIndex = 10;
            this.TextButton.Text = "Text";
            this.TextButton.Click += new System.EventHandler(this.TextButton_Click);
            // 
            // groupBox1
            // 
            this.groupBox1.Controls.Add(this.RectangleButton);
            this.groupBox1.Controls.Add(this.CircleButton);
            this.groupBox1.Controls.Add(this.EraseButton);
            this.groupBox1.Controls.Add(this.ROIButton);
            this.groupBox1.Controls.Add(this.LineButton);
            this.groupBox1.Location = new System.Drawing.Point(19, 18);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(336, 120);
            this.groupBox1.TabIndex = 5;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Graphic Overlays";
            // 
            // RectangleButton
            // 
            this.RectangleButton.Location = new System.Drawing.Point(19, 28);
            this.RectangleButton.Name = "RectangleButton";
            this.RectangleButton.Size = new System.Drawing.Size(90, 26);
            this.RectangleButton.TabIndex = 0;
            this.RectangleButton.Text = "Rectangle";
            this.RectangleButton.Click += new System.EventHandler(this.RectangleButton_Click);
            // 
            // CircleButton
            // 
            this.CircleButton.Location = new System.Drawing.Point(19, 74);
            this.CircleButton.Name = "CircleButton";
            this.CircleButton.Size = new System.Drawing.Size(90, 26);
            this.CircleButton.TabIndex = 6;
            this.CircleButton.Text = "Circle";
            this.CircleButton.Click += new System.EventHandler(this.CircleButton_Click);
            // 
            // EraseButton
            // 
            this.EraseButton.Location = new System.Drawing.Point(125, 46);
            this.EraseButton.Name = "EraseButton";
            this.EraseButton.Size = new System.Drawing.Size(90, 27);
            this.EraseButton.TabIndex = 8;
            this.EraseButton.Text = "Erase";
            this.EraseButton.Click += new System.EventHandler(this.EraseButton_Click);
            // 
            // ROIButton
            // 
            this.ROIButton.Location = new System.Drawing.Point(230, 74);
            this.ROIButton.Name = "ROIButton";
            this.ROIButton.Size = new System.Drawing.Size(90, 26);
            this.ROIButton.TabIndex = 7;
            this.ROIButton.Text = "ROI";
            this.ROIButton.Click += new System.EventHandler(this.ROIButton_Click);
            // 
            // LineButton
            // 
            this.LineButton.Location = new System.Drawing.Point(230, 28);
            this.LineButton.Name = "LineButton";
            this.LineButton.Size = new System.Drawing.Size(90, 26);
            this.LineButton.TabIndex = 9;
            this.LineButton.Text = "Line";
            this.LineButton.Click += new System.EventHandler(this.LineButton_Click);
            // 
            // lineThickness
            // 
            this.lineThickness.Location = new System.Drawing.Point(576, 24);
            this.lineThickness.Name = "lineThickness";
            this.lineThickness.Size = new System.Drawing.Size(112, 56);
            this.lineThickness.TabIndex = 14;
            this.lineThickness.TabStop = false;
            this.lineThickness.Text = "Line Thickness";
            // 
            // groupBox2
            // 
            this.groupBox2.Controls.Add(this.RectFrameRadioButton);
            this.groupBox2.Controls.Add(this.EllipseFrameRadioButton);
            this.groupBox2.Location = new System.Drawing.Point(368, 16);
            this.groupBox2.Name = "groupBox2";
            this.groupBox2.Size = new System.Drawing.Size(192, 120);
            this.groupBox2.TabIndex = 18;
            this.groupBox2.TabStop = false;
            this.groupBox2.Text = "Graphics Special Effects";
            // 
            // RectFrameRadioButton
            // 
            this.RectFrameRadioButton.Location = new System.Drawing.Point(16, 32);
            this.RectFrameRadioButton.Name = "RectFrameRadioButton";
            this.RectFrameRadioButton.Size = new System.Drawing.Size(144, 25);
            this.RectFrameRadioButton.TabIndex = 17;
            this.RectFrameRadioButton.Text = "Rectangular Frame";
            this.RectFrameRadioButton.CheckedChanged += new System.EventHandler(this.RectFrameRadioButton_CheckedChanged);
            // 
            // EllipseFrameRadioButton
            // 
            this.EllipseFrameRadioButton.Location = new System.Drawing.Point(16, 72);
            this.EllipseFrameRadioButton.Name = "EllipseFrameRadioButton";
            this.EllipseFrameRadioButton.Size = new System.Drawing.Size(136, 24);
            this.EllipseFrameRadioButton.TabIndex = 16;
            this.EllipseFrameRadioButton.Text = "Ellipsoidal Frame";
            this.EllipseFrameRadioButton.CheckedChanged += new System.EventHandler(this.EllipseFrameRadioButton_CheckedChanged);
            // 
            // tabPage1
            // 
            this.tabPage1.Controls.Add(this.redEyeButton);
            this.tabPage1.Controls.Add(this.groupBox4);
            this.tabPage1.Controls.Add(this.Brightness);
            this.tabPage1.Controls.Add(this.IPResetButton);
            this.tabPage1.Controls.Add(this.ApplyProcessingButton);
            this.tabPage1.Controls.Add(this.groupBox3);
            this.tabPage1.Controls.Add(this.label1);
            this.tabPage1.Controls.Add(this.picturePreView);
            this.tabPage1.Location = new System.Drawing.Point(4, 25);
            this.tabPage1.Name = "tabPage1";
            this.tabPage1.Size = new System.Drawing.Size(835, 187);
            this.tabPage1.TabIndex = 2;
            this.tabPage1.Text = "Image Processing";
            // 
            // redEyeButton
            // 
            this.redEyeButton.Location = new System.Drawing.Point(664, 120);
            this.redEyeButton.Name = "redEyeButton";
            this.redEyeButton.Size = new System.Drawing.Size(75, 40);
            this.redEyeButton.TabIndex = 7;
            this.redEyeButton.Text = "Remove Red Eye";
            this.redEyeButton.Click += new System.EventHandler(this.redEyeButton_Click);
            // 
            // groupBox4
            // 
            this.groupBox4.Controls.Add(this.label4);
            this.groupBox4.Controls.Add(this.label3);
            this.groupBox4.Controls.Add(this.label2);
            this.groupBox4.Controls.Add(this.trackBarRed);
            this.groupBox4.Controls.Add(this.trackBarBlue);
            this.groupBox4.Controls.Add(this.trackBarGreen);
            this.groupBox4.Location = new System.Drawing.Point(168, 8);
            this.groupBox4.Name = "groupBox4";
            this.groupBox4.Size = new System.Drawing.Size(272, 175);
            this.groupBox4.TabIndex = 6;
            this.groupBox4.TabStop = false;
            this.groupBox4.Text = "Color";
            // 
            // label4
            // 
            this.label4.Location = new System.Drawing.Point(184, 24);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(80, 23);
            this.label4.TabIndex = 5;
            this.label4.Text = "Blue";
            // 
            // label3
            // 
            this.label3.Location = new System.Drawing.Point(96, 24);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(48, 23);
            this.label3.TabIndex = 4;
            this.label3.Text = "Green";
            // 
            // label2
            // 
            this.label2.Location = new System.Drawing.Point(24, 24);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(32, 23);
            this.label2.TabIndex = 3;
            this.label2.Text = "Red";
            // 
            // trackBarRed
            // 
            this.trackBarRed.LargeChange = 10;
            this.trackBarRed.Location = new System.Drawing.Point(24, 40);
            this.trackBarRed.Maximum = 100;
            this.trackBarRed.Minimum = -100;
            this.trackBarRed.Name = "trackBarRed";
            this.trackBarRed.Orientation = System.Windows.Forms.Orientation.Vertical;
            this.trackBarRed.Size = new System.Drawing.Size(56, 128);
            this.trackBarRed.TabIndex = 2;
            this.trackBarRed.TickFrequency = 10;
            this.trackBarRed.Scroll += new System.EventHandler(this.trackBarRed_Scroll);
            // 
            // trackBarBlue
            // 
            this.trackBarBlue.LargeChange = 10;
            this.trackBarBlue.Location = new System.Drawing.Point(184, 40);
            this.trackBarBlue.Maximum = 100;
            this.trackBarBlue.Minimum = -100;
            this.trackBarBlue.Name = "trackBarBlue";
            this.trackBarBlue.Orientation = System.Windows.Forms.Orientation.Vertical;
            this.trackBarBlue.Size = new System.Drawing.Size(56, 128);
            this.trackBarBlue.TabIndex = 1;
            this.trackBarBlue.TickFrequency = 10;
            this.trackBarBlue.Scroll += new System.EventHandler(this.trackBarBlue_Scroll);
            // 
            // trackBarGreen
            // 
            this.trackBarGreen.LargeChange = 10;
            this.trackBarGreen.Location = new System.Drawing.Point(96, 40);
            this.trackBarGreen.Maximum = 100;
            this.trackBarGreen.Minimum = -100;
            this.trackBarGreen.Name = "trackBarGreen";
            this.trackBarGreen.Orientation = System.Windows.Forms.Orientation.Vertical;
            this.trackBarGreen.Size = new System.Drawing.Size(56, 128);
            this.trackBarGreen.TabIndex = 0;
            this.trackBarGreen.TickFrequency = 10;
            this.trackBarGreen.Scroll += new System.EventHandler(this.trackBarGreen_Scroll);
            // 
            // Brightness
            // 
            this.Brightness.Controls.Add(this.trackBarBrightness);
            this.Brightness.Location = new System.Drawing.Point(16, 96);
            this.Brightness.Name = "Brightness";
            this.Brightness.Size = new System.Drawing.Size(144, 87);
            this.Brightness.TabIndex = 5;
            this.Brightness.TabStop = false;
            this.Brightness.Text = "Brightness";
            // 
            // trackBarBrightness
            // 
            this.trackBarBrightness.LargeChange = 10;
            this.trackBarBrightness.Location = new System.Drawing.Point(8, 24);
            this.trackBarBrightness.Maximum = 300;
            this.trackBarBrightness.Minimum = -300;
            this.trackBarBrightness.Name = "trackBarBrightness";
            this.trackBarBrightness.Size = new System.Drawing.Size(128, 56);
            this.trackBarBrightness.TabIndex = 0;
            this.trackBarBrightness.TickFrequency = 30;
            this.trackBarBrightness.Scroll += new System.EventHandler(this.trackBarBrightness_Scroll);
            // 
            // IPResetButton
            // 
            this.IPResetButton.Location = new System.Drawing.Point(664, 80);
            this.IPResetButton.Name = "IPResetButton";
            this.IPResetButton.TabIndex = 4;
            this.IPResetButton.Text = "Reset";
            this.IPResetButton.Click += new System.EventHandler(this.IPResetButton_Click);
            // 
            // ApplyProcessingButton
            // 
            this.ApplyProcessingButton.Location = new System.Drawing.Point(664, 40);
            this.ApplyProcessingButton.Name = "ApplyProcessingButton";
            this.ApplyProcessingButton.TabIndex = 3;
            this.ApplyProcessingButton.Text = "Apply";
            this.ApplyProcessingButton.Click += new System.EventHandler(this.ApplyProcessingButton_Click);
            // 
            // groupBox3
            // 
            this.groupBox3.Controls.Add(this.trackBarContrast);
            this.groupBox3.Location = new System.Drawing.Point(16, 8);
            this.groupBox3.Name = "groupBox3";
            this.groupBox3.Size = new System.Drawing.Size(144, 80);
            this.groupBox3.TabIndex = 2;
            this.groupBox3.TabStop = false;
            this.groupBox3.Text = "Contrast";
            // 
            // trackBarContrast
            // 
            this.trackBarContrast.Cursor = System.Windows.Forms.Cursors.Hand;
            this.trackBarContrast.LargeChange = 10;
            this.trackBarContrast.Location = new System.Drawing.Point(8, 16);
            this.trackBarContrast.Maximum = 100;
            this.trackBarContrast.Minimum = -100;
            this.trackBarContrast.Name = "trackBarContrast";
            this.trackBarContrast.Size = new System.Drawing.Size(128, 56);
            this.trackBarContrast.TabIndex = 0;
            this.trackBarContrast.TickFrequency = 10;
            this.trackBarContrast.Scroll += new System.EventHandler(this.trackBarContrast_Scroll);
            // 
            // label1
            // 
            this.label1.Location = new System.Drawing.Point(456, 16);
            this.label1.Name = "label1";
            this.label1.TabIndex = 1;
            this.label1.Text = "Preview";
            // 
            // picturePreView
            // 
            this.picturePreView.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.picturePreView.Location = new System.Drawing.Point(456, 40);
            this.picturePreView.Name = "picturePreView";
            this.picturePreView.Size = new System.Drawing.Size(144, 120);
            this.picturePreView.TabIndex = 0;
            this.picturePreView.TabStop = false;
            // 
            // tabPage2
            // 
            this.tabPage2.Controls.Add(this.placementButton);
            this.tabPage2.Controls.Add(this.textUndoButton);
            this.tabPage2.Controls.Add(this.textApplyButton);
            this.tabPage2.Controls.Add(this.label8);
            this.tabPage2.Controls.Add(this.label5);
            this.tabPage2.Controls.Add(this.rotationX);
            this.tabPage2.Controls.Add(this.colorButton);
            this.tabPage2.Controls.Add(this.fontButton);
            this.tabPage2.Controls.Add(this.textBox);
            this.tabPage2.Controls.Add(this.rotationY);
            this.tabPage2.Controls.Add(this.rotationZ);
            this.tabPage2.Controls.Add(this.label6);
            this.tabPage2.Controls.Add(this.label7);
            this.tabPage2.Controls.Add(this.groupBox5);
            this.tabPage2.Location = new System.Drawing.Point(4, 25);
            this.tabPage2.Name = "tabPage2";
            this.tabPage2.Size = new System.Drawing.Size(835, 187);
            this.tabPage2.TabIndex = 3;
            this.tabPage2.Text = "3D Text";
            // 
            // placementButton
            // 
            this.placementButton.Location = new System.Drawing.Point(38, 129);
            this.placementButton.Name = "placementButton";
            this.placementButton.Size = new System.Drawing.Size(192, 27);
            this.placementButton.TabIndex = 9;
            this.placementButton.Text = "Select Placement";
            this.placementButton.Click += new System.EventHandler(this.placementButton_Click);
            // 
            // textUndoButton
            // 
            this.textUndoButton.Location = new System.Drawing.Point(490, 92);
            this.textUndoButton.Name = "textUndoButton";
            this.textUndoButton.Size = new System.Drawing.Size(90, 27);
            this.textUndoButton.TabIndex = 8;
            this.textUndoButton.Text = "Undo";
            this.textUndoButton.Click += new System.EventHandler(this.textUndoButton_Click);
            // 
            // textApplyButton
            // 
            this.textApplyButton.Location = new System.Drawing.Point(490, 46);
            this.textApplyButton.Name = "textApplyButton";
            this.textApplyButton.Size = new System.Drawing.Size(90, 27);
            this.textApplyButton.TabIndex = 7;
            this.textApplyButton.Text = "Apply";
            this.textApplyButton.Click += new System.EventHandler(this.textApplyButton_Click);
            // 
            // label8
            // 
            this.label8.Location = new System.Drawing.Point(38, 18);
            this.label8.Name = "label8";
            this.label8.Size = new System.Drawing.Size(120, 19);
            this.label8.TabIndex = 6;
            this.label8.Text = "Text";
            // 
            // label5
            // 
            this.label5.Location = new System.Drawing.Point(307, 37);
            this.label5.Name = "label5";
            this.label5.Size = new System.Drawing.Size(19, 18);
            this.label5.TabIndex = 4;
            this.label5.Text = "X:";
            // 
            // rotationX
            // 
            this.rotationX.Location = new System.Drawing.Point(346, 37);
            this.rotationX.Maximum = new System.Decimal(new int[] {
                                                                      180,
                                                                      0,
                                                                      0,
                                                                      0});
            this.rotationX.Minimum = new System.Decimal(new int[] {
                                                                      180,
                                                                      0,
                                                                      0,
                                                                      -2147483648});
            this.rotationX.Name = "rotationX";
            this.rotationX.Size = new System.Drawing.Size(67, 22);
            this.rotationX.TabIndex = 3;
            // 
            // colorButton
            // 
            this.colorButton.Location = new System.Drawing.Point(144, 83);
            this.colorButton.Name = "colorButton";
            this.colorButton.Size = new System.Drawing.Size(90, 27);
            this.colorButton.TabIndex = 2;
            this.colorButton.Text = "Color...";
            this.colorButton.Click += new System.EventHandler(this.colorButton_Click);
            // 
            // fontButton
            // 
            this.fontButton.Location = new System.Drawing.Point(38, 83);
            this.fontButton.Name = "fontButton";
            this.fontButton.Size = new System.Drawing.Size(90, 27);
            this.fontButton.TabIndex = 1;
            this.fontButton.Text = "Font...";
            this.fontButton.Click += new System.EventHandler(this.fontButton_Click);
            // 
            // textBox
            // 
            this.textBox.Location = new System.Drawing.Point(38, 37);
            this.textBox.Name = "textBox";
            this.textBox.Size = new System.Drawing.Size(202, 22);
            this.textBox.TabIndex = 0;
            this.textBox.Text = "Enter text here...";
            // 
            // rotationY
            // 
            this.rotationY.Location = new System.Drawing.Point(346, 74);
            this.rotationY.Maximum = new System.Decimal(new int[] {
                                                                      180,
                                                                      0,
                                                                      0,
                                                                      0});
            this.rotationY.Minimum = new System.Decimal(new int[] {
                                                                      180,
                                                                      0,
                                                                      0,
                                                                      -2147483648});
            this.rotationY.Name = "rotationY";
            this.rotationY.Size = new System.Drawing.Size(67, 22);
            this.rotationY.TabIndex = 3;
            // 
            // rotationZ
            // 
            this.rotationZ.Location = new System.Drawing.Point(346, 111);
            this.rotationZ.Maximum = new System.Decimal(new int[] {
                                                                      180,
                                                                      0,
                                                                      0,
                                                                      0});
            this.rotationZ.Minimum = new System.Decimal(new int[] {
                                                                      180,
                                                                      0,
                                                                      0,
                                                                      -2147483648});
            this.rotationZ.Name = "rotationZ";
            this.rotationZ.Size = new System.Drawing.Size(67, 22);
            this.rotationZ.TabIndex = 3;
            // 
            // label6
            // 
            this.label6.Location = new System.Drawing.Point(307, 74);
            this.label6.Name = "label6";
            this.label6.Size = new System.Drawing.Size(19, 18);
            this.label6.TabIndex = 4;
            this.label6.Text = "Y:";
            // 
            // label7
            // 
            this.label7.Location = new System.Drawing.Point(307, 111);
            this.label7.Name = "label7";
            this.label7.Size = new System.Drawing.Size(19, 18);
            this.label7.TabIndex = 4;
            this.label7.Text = "Z:";
            // 
            // groupBox5
            // 
            this.groupBox5.Location = new System.Drawing.Point(269, 9);
            this.groupBox5.Name = "groupBox5";
            this.groupBox5.Size = new System.Drawing.Size(182, 148);
            this.groupBox5.TabIndex = 5;
            this.groupBox5.TabStop = false;
            this.groupBox5.Text = "Rotation";
            // 
            // fontDialog
            // 
            this.fontDialog.Font = new System.Drawing.Font("Microsoft Sans Serif", 32F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
            // 
            // colorDialog
            // 
            this.colorDialog.Color = System.Drawing.Color.Red;
            // 
            // PhotoEditorForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(6, 15);
            this.ClientSize = new System.Drawing.Size(843, 683);
            this.Controls.Add(this.tabControl);
            this.Controls.Add(this.customScrollableControl);
            this.Menu = this.MainMenu;
            this.Name = "PhotoEditorForm";
            this.Text = "Photo Editor Application";
            this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
            this.tabControl.ResumeLayout(false);
            this.basicImageOperations.ResumeLayout(false);
            this.Overlays.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.penNumericUpDown)).EndInit();
            this.groupBox1.ResumeLayout(false);
            this.groupBox2.ResumeLayout(false);
            this.tabPage1.ResumeLayout(false);
            this.groupBox4.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.trackBarRed)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarBlue)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.trackBarGreen)).EndInit();
            this.Brightness.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.trackBarBrightness)).EndInit();
            this.groupBox3.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.trackBarContrast)).EndInit();
            this.tabPage2.ResumeLayout(false);
            ((System.ComponentModel.ISupportInitialize)(this.rotationX)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.rotationY)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.rotationZ)).EndInit();
            this.ResumeLayout(false);

        }
        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main() 
        {
            Application.Run(new PhotoEditorForm());
        }

        /// <summary>
        /// Called if the Load menu item was selected. Calls LoadImage on Picture class.
        /// </summary>
        private void OpenFile_Click(object sender, System.EventArgs e)
        {
            try
            {
                PictureObject.LoadImage();
                if(PictureObject.LoadedImage == null) 
                    throw(new Exception("Error, image could not beloaded"));
                // Create a thumbnail image from the loaded image.
                // The image will be displayed in the preview of the 
                // Image processing tab.
                thumbImage = PictureObject.LoadedImage.GetThumbnailImage( 
                    picturePreView.Width, 
                    picturePreView.Height,
                    null, 
                    IntPtr.Zero);

                picturePreView.Image = thumbImage;

                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }

        }
        /// <summary>
        /// Method used to set the image displayed in the
        /// custom control to the current picture object image.
        /// Invalidates the custom control. 
        /// </summary>
        public void DisplayImage()
        {
            customScrollableControl.ScrollingImage = PictureObject.LoadedImage;
            customScrollableControl.Invalidate();
            PictureObject.Draw();
        }
       
        /// <summary>
        /// Opens theCrop Dialog box window to allow the user to specify
        /// a new image size. The image will be cropped to the width and
        /// height where the cropped image is taken from the middle of the 
        /// original image.
        /// </summary>
        /// <param name="sender">A reference to the object calling this method</param>
        /// <param name="e">The event arguments provided by the event handler</param>
        private void cropImageButton_Click(object sender, System.EventArgs e)
        {
            try
            {
                CropDialog openDialog = new CropDialog(this);
                if(openDialog == null) 
                    throw(new Exception("Error, CropDialog could not be allocated"));
                openDialog.Show();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }
        
        /// <summary>
        /// Calls the RoatateFlipImage method of the Image class
        /// </summary>
        private void flipHorizontalButton_Click(object sender, System.EventArgs e)
        {
            try
            {
                PictureObject.RotateFlipImage(RotateFlipType.RotateNoneFlipX);
                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        
        }
        /// <summary>
        /// Calls the RotateFlipImage method of the Image class
        /// </summary>
        private void flipVerticalButton_Click(object sender, System.EventArgs e)
        {
            try
            {
                PictureObject.RotateFlipImage(RotateFlipType.RotateNoneFlipY);
                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }
        /// <summary>
        /// Calls the RotateFlipImage method of the Image class
        /// </summary>
        private void rotateClockButton_Click(object sender, System.EventArgs e)
        {
            try
            {
                PictureObject.RotateFlipImage(RotateFlipType.Rotate90FlipNone);
                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        
        }
        /// <summary>
        /// Calls the RotateFlipImage method of the Image class
        /// </summary>
        private void rotateCClockButton_Click(object sender, System.EventArgs e)
        {
            try
            {
                PictureObject.RotateFlipImage(RotateFlipType.Rotate270FlipNone);
                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }
        
        /// <summary>
        /// Calls the SaveImage() method of the Image class
        /// </summary>
        private void saveMenu_Click(object sender, System.EventArgs e)
        {   
            try
            {
                PictureObject.SaveImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Convenience method to reset all buttons to
        /// default appearance and to reset toolSelected.
        /// </summary>
        private void resetButtons()
        {
            // Resets all the implemented graphic tool buttons 
            RectangleButton.FlatStyle = FlatStyle.Standard;
            TextButton.FlatStyle = FlatStyle.Standard;
            LineButton.FlatStyle = FlatStyle.Standard;
            CircleButton.FlatStyle = FlatStyle.Standard;
            ROIButton.FlatStyle = FlatStyle.Standard;
            EraseButton.FlatStyle = FlatStyle.Standard;
            RectFrameRadioButton.Checked = false;
            EllipseFrameRadioButton.Checked = false;

            // Resets the tool currently used to none.
            toolSelected = toolSelection.None;

           
        }

        // All possible tools that can be selected
        private enum toolSelection { None = 0, RectangleTool, 
            LineTool, CircleTool, ROITool, TextTool, SelectTool,
        EllipseFrameTool, RectFrameTool, RedEyeTool}
        private bool mouseInControl = false;
        // Currently tool selected
        private toolSelection toolSelected;

        /// <summary>
        /// Sets the selected tool to RactangleTool if no tool
        /// was selected before. If a tool was selected before
        /// then the tool selection is reset to no tool.
        /// </summary>
        /// <param name="sender">Object that sent the event.</param>
        /// <param name="e">Parameters provided by the event.</param>
        private void RectangleButton_Click(object sender, System.EventArgs e)
        {
            if(toolSelected == toolSelection.None)
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
                toolSelected = toolSelection.RectangleTool;
                // Changes the appearance of the button to show it 
                // is actually selected
                RectangleButton.FlatStyle = FlatStyle.Flat;
            }
            else 
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
            }  
        }

        // Indicates if a tool was selected, the mouse button was 
        // selected
        // and is not released yet => currently drawing the selected 
        // component
        private bool isDrawing;

        // The selected component that is currently drawing
        private GraphicsComponent compObject;
        private GraphicsComponent regionComponent;
        // If tool was selected and mouse is clicked in the window 
        // then
        // create the component initialize the variables and draw 
        // everything.
        private void customScrollableControl_MouseDown(
            object sender, 
            System.Windows.Forms.MouseEventArgs e)
        {
            // Just go here if tool is not textTool
            if((toolSelected != toolSelection.TextTool) && (toolSelected != toolSelection.RedEyeTool))
            {
                // Check if tool is selected and drawing and it is not the select tool.
                if(!isDrawing && toolSelected != 0)
                {
                    switch(toolSelected)
                    {
                            //create component according to tool selected
                        case toolSelection.LineTool:
                            compObject = new LineComponent();
                            break;
                        case toolSelection.RectangleTool:
                            compObject = new RectangleComponent();
                            break;
                        case toolSelection.CircleTool:
                            compObject = new CircleComponent();
                            break;
                        case toolSelection.ROITool:
                            compObject = new ROIComponent();
                            regionComponent = compObject;
                            break;
                        case toolSelection.SelectTool:
                            compObject = null;
                            compObject = (GraphicsComponent)PictureObject.PositionIsComponent(e.X + 
                                customScrollableControl.ViewportCoords().X, 
                                e.Y + customScrollableControl.ViewportCoords().Y);
                            if(compObject != null)
                            {
                                PictureObject.Remove(compObject);
                            }
                            break;
                        case toolSelection.RectFrameTool:
                            compObject = new RectFrameComponent();
                            break;
                        case toolSelection.EllipseFrameTool:
                            compObject = new EllipseFrameComponent();
                            break;
                        default:
                            toolSelected = toolSelection.None;
                            throw(new Exception("Error, Tool selected not implemented"));
                    }
                    if((toolSelected != toolSelection.SelectTool) && (toolSelected != toolSelection.None))
                    {
                        // Set the Graphics referenced in the component to the window object.
                        GraphicsComponent.DeviceContext = customScrollableControl.CreateGraphics();
                        // Add the component to the list that needs to be drawn.
                        PictureObject.Add(compObject);
                        // Get the coordinates and transfer them to coordinates relative to the image
                        Point tempPoint = new Point(e.X + customScrollableControl.ViewportCoords().X, 
                            e.Y + customScrollableControl.ViewportCoords().Y);
                        // Set the points
                        compObject.StartPoint = tempPoint;
                        compObject.EndPoint = tempPoint;
					
                        // Now we started drawing a component
                        isDrawing = true;
                    }
                    // Update the pen which is used to draw the 
                    // components
                    GraphicsComponent.DrawingPen = selectedPen;

                    // Display the components.
                    PictureObject.Draw();
                }
            }
            else if(toolSelected == toolSelection.RedEyeTool)
            {
				
                Graphics myDC = customScrollableControl.CreateGraphics();
					
                DisplayImage();
                PictureObject.Draw();
					
                plugInInterface.SliderName = "RedEye";
                plugInInterface.Param1 = e.X + customScrollableControl.ViewportCoords().X;
                plugInInterface.Param2 = e.Y + customScrollableControl.ViewportCoords().Y;
                plugInInterface.Param3 = 10;
                myDC.DrawRectangle(Pens.Red, plugInInterface.Param1, plugInInterface.Param2, 10, 10);
                plugInInterface.TmpBitmap = PictureObject.LoadedImage;
                applyImageProcessing();
				
            }
            else
            {
                // Create component
                compObject = new TextComponent();
                // Set Graphics object to the customScrollableControl
                GraphicsComponent.DeviceContext = customScrollableControl.CreateGraphics();
                Point tempPoint = new Point(e.X + customScrollableControl.ViewportCoords().X, e.Y + customScrollableControl.ViewportCoords().Y);
                compObject.StartPoint = tempPoint;
                compObject.EndPoint = tempPoint;
                // Show the text input box
                inputTextBox.Show();
                // Register the callback function
                inputTextBox.CallMethod(new TextBoxInput.Feedback(MyEventHandler));	
            }
        }

        // Delegate example for callback function.
        void MyEventHandler(string text)
        {
            TextComponent texC = (TextComponent) compObject;
            texC.TextInput = inputTextBox.TextInput;

            //Add Component to list
            PictureObject.Add(compObject);
			
            //Display everything
            DisplayImage();
            PictureObject.Draw();
        }
    
        // Event Handler for mouse move, 
        // draws component when in draw mode.
        private void customScrollableControl_MouseMove(object 
            sender, System.Windows.Forms.MouseEventArgs e)
        {
            // Check if there was a tool selected.
            if(toolSelected != 0)
            {
                // Change cursor to cross shape.
                Cursor.Current = Cursors.Cross;
            }

            // Check if in drawing mode.
            if(isDrawing)
            {
                // Set new coordinate of endpoint for component 
                // in 
                // coordinates relative to the image.
                Point tempPoint = new Point(e.X + customScrollableControl.ViewportCoords().X, e.Y + customScrollableControl.ViewportCoords().Y);
                compObject.EndPoint = tempPoint;
                
                // Draw everything
                DisplayImage();
                PictureObject.Draw();
            }
            // If the redEye tool is selected
            // draw a rectangle at the current position.
            if(toolSelected == toolSelection.RedEyeTool)
            {
                Graphics myDC = customScrollableControl.CreateGraphics();
                // Get coordinates of mouse.
                // The Coordinates are set in MouseMove event handler.
                plugInInterface.Param1 = e.X;
                plugInInterface.Param2 = e.Y;
                DisplayImage();
                PictureObject.Draw();
                // Draw a red rectangle at the current cursor position.
                myDC.DrawRectangle(Pens.Red, e.X, e.Y, 10, 10);

				
            }
            if(toolSelected == toolSelection.SelectTool)
            {
                const int delta = 10;
                compObject = null;
                compObject = (GraphicsComponent)PictureObject.PositionIsComponent(e.X + customScrollableControl.ViewportCoords().X, 
                    e.Y + customScrollableControl.ViewportCoords().Y);
				
                if(compObject != null)
                {
                    // if an object is picked, then draw the red rectagles around its start and end points to indicate
                    // the picking
                    Graphics myDC = customScrollableControl.CreateGraphics();
                    Point tempPoint = new Point(compObject.StartPoint.X - 5, compObject.StartPoint.Y - 5);
                    tempPoint = compObject.CalcViewportCoords(tempPoint);
                    myDC.DrawRectangle(Pens.Red, tempPoint.X, tempPoint.Y, delta, delta);

                    tempPoint.X = compObject.EndPoint.X - 5;
                    tempPoint.Y = compObject.EndPoint.Y - 5;
                    tempPoint = compObject.CalcViewportCoords(tempPoint);
                    myDC.DrawRectangle(Pens.Red, tempPoint.X, tempPoint.Y, delta, delta);
					
                }
                else
                {
                    // Draw everything
                    DisplayImage();
                    PictureObject.Draw();
                }
            }
        }

        // If mouse button is released and drawing mode then 
        // add object to list and redraw everything
        private void customScrollableControl_MouseUp(object 
            sender, System.Windows.Forms.MouseEventArgs e)
        {
            if(isDrawing)
            {
                // Update endpoint to final position relative to image 
                Point tempPoint = new Point(e.X + customScrollableControl.ViewportCoords().X, e.Y + customScrollableControl.ViewportCoords().Y);
                compObject.EndPoint = tempPoint;
                
                // If ROI tool remove now from List...
                if(toolSelected == toolSelection.ROITool)
                {
                    PictureObject.Remove(compObject);
                }

                // Reset mode to not drawing
                isDrawing = false;

                // Display all
                DisplayImage();
                PictureObject.Draw();
            }
        }

    
        // If mouse enters control and a tool is selected
        // than change the cursor to cross
        private void customScrollableControl_MouseEnter(object 
            sender, System.EventArgs e)
        {
            mouseInControl = true;
            if(toolSelected != 0)
            {
                Cursor.Current = Cursors.Cross;
            }
        }

        // If mouse leaves control make sure 
        // mouse pointer is standard
        private void customScrollableControl_MouseLeave(
            object sender, System.EventArgs e)
        {
            mouseInControl = false;
            Cursor.Current = Cursors.Default;
            DisplayImage();
        }

        /// <summary>
        /// Text Button was clicked and dialog is poped up for text input.
        /// </summary>
        private void TextButton_Click(object sender, System.EventArgs e)
        {
            if(toolSelected == toolSelection.None)
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
                toolSelected = toolSelection.TextTool;
                // Changes the appearance of the button to show it 
                // is actually selected
                TextButton.FlatStyle = FlatStyle.Flat;
            }
            else 
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
            }  
        
        }

        /// <summary>
        /// Overwrites the pait method. Updates the viewport coords and draws the graphic overlays.
        /// </summary>
        private void customScrollableControl_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            // First update the viewport Coordinates.
            GraphicsComponent.ViewportCoords = customScrollableControl.ViewportCoords();
            // Draw all graphic components.
            PictureObject.Draw();
            if((toolSelected == toolSelection.RedEyeTool) && mouseInControl)
            {
                Graphics myDC = customScrollableControl.CreateGraphics();
                Point tempPoint = new Point(plugInInterface.Param1, plugInInterface.Param2);
                myDC.DrawRectangle(Pens.Red, plugInInterface.Param1, plugInInterface.Param2, 10, 10);
            }
            
        }
        /// <summary>
        /// Line tool is selected.
        /// </summary>
        private void LineButton_Click(object sender, System.EventArgs e)
        {
            if(toolSelected == toolSelection.None)
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
                toolSelected = toolSelection.LineTool;
                // Changes the appearance of the button to show it 
                // is actually selected
                LineButton.FlatStyle = FlatStyle.Flat;
            }
            else 
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
            }  
        
        }

        /// <summary>
        /// Circle tool is selected.
        /// </summary>
        private void CircleButton_Click(object sender, System.EventArgs e)
        {
            if(toolSelected == toolSelection.None)
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
                toolSelected = toolSelection.CircleTool;
                // Changes the appearance of the button to show it 
                // is actually selected
                CircleButton.FlatStyle = FlatStyle.Flat;
            }
            else 
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
            }  
        
        }

        /// <summary>
        /// ROI tool selected.
        /// </summary>
        private void ROIButton_Click(object sender, System.EventArgs e)
        {
            if(toolSelected == toolSelection.None)
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
                toolSelected = toolSelection.ROITool;
                // Changes the appearance of the button to show it 
                // is actually selected
                ROIButton.FlatStyle = FlatStyle.Flat;
            }
            else 
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
            }  
        
        }

        /// <summary>
        /// Erase button sets selection tool.
        /// </summary>
        private void EraseButton_Click(object sender, System.EventArgs e)
        {
            if(toolSelected == toolSelection.None)
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
                toolSelected = toolSelection.SelectTool;
                // Changes the appearance of the button to show it 
                // is actually selected
               EraseButton.FlatStyle = FlatStyle.Flat;
            }
            else 
            {
                // Calls the convenience method to 
                // reset the buttons to no tool selected.
                resetButtons();
            }  
        
        }

        /// <summary>
        /// Applies the graphic overlays by burning them into the image. 
        /// Burning done
        /// in picture class.
        /// </summary>
        private void ApplyButton_Click(object sender, System.EventArgs e)
        {
            PictureObject.ApplyPictureComponents();
            resetButtons();
        }

        /// <summary>
        /// Reset button, resets to the original loaded image. 
        /// Functionality in Picture class is called.
        /// </summary>
        private void ResetButton_Click(object sender, System.EventArgs e)
        {
            PictureObject.Reset();
            DisplayImage();
        }
             
        /// <summary>
        /// Exit button pressed, close application.
        /// </summary>
        private void menuItem2_Click(object sender, System.EventArgs e)
        {
            // Just get out of here....
            this.Close();
        }

        private void customScrollableControl_Load(object sender, System.EventArgs e)
        {
            DisplayImage();
        }

        /// <summary>
        /// Event handler for the NumericUpDown control which
        /// enables the user to select the line width of the pen.
        /// </summary>
        private void penNumericUpDown_ValueChanged(object sender, System.EventArgs e)
        {
            try
            {
                selectedPen.Width = (float)(penNumericUpDown.Value);
                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Pen used to draw the components. Default Pen
        /// is of width = 1 and color = black.
        /// </summary>
        private Pen selectedPen = new Pen(Color.Black, 1);

        /// <summary>
        /// Opens a color dialog box to enable the user
        /// to select a color for the pen.
        /// </summary>
        private void selectColorButton_Click(object sender, System.EventArgs e)
        {
            try
            {
                // Opens and shows a color dialog box.
                ColorDialog myDialog = new ColorDialog();
                myDialog.ShowDialog();
                // Sets the pen color to the selected color.
                selectedPen.Color = myDialog.Color;
                // Delete the color dialog.
                // Not really necessary because garbage collection
                // will take care of it anyway.
                myDialog.Dispose();
                DisplayImage();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Sets the currently eslected tool to 
        /// the RectangleFrameTool.
        /// </summary> 
        /// <requirements>F:image_graphics_special_effects</requirements>
        private void RectFrameRadioButton_CheckedChanged(object sender, System.EventArgs e)
        {
            try
            {               
                toolSelected = toolSelection.RectFrameTool;    
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Sets the currently eslected tool to 
        /// the EllipseFrameTool.
        /// </summary>
        ///<requirements>F:image_graphics_special_effects
        /// </requirements>
        private void EllipseFrameRadioButton_CheckedChanged(object sender, System.EventArgs e)
        {
            try
            {
               toolSelected = toolSelection.EllipseFrameTool;   
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }		
        }
        /// <summary>
        /// This method is called when a new thumbnail is
        /// calculated for preview a transformation.
        /// </summary>
        private void applyImageProcessing()
        {
            // Copy the current thumbnail to the tmpBitmap
            // which is used as the image on which the
            // image processing calculation is performed.
            plugInInterface.TmpBitmap = (Bitmap)thumbImage.Clone();
            
            // Call the PlugInInterface method to do the magic.
            plugInInterface.ApplyImageProcessing();
            
            // Set the preview Image to a copy of the calculated tmpImage.
            picturePreView.Image = (Image)plugInInterface.TmpBitmap.Clone();  
        }

        private Image thumbImage;
        /// <summary>
        /// Event handler for the scroll event of 
        /// the contrast control. Calls 
        /// applyImageProcessing(...)
        /// </summary>
        /// <param name="sender">Sender object.</param>
        /// <param name="e">Event arguments.</param>
        /// <requirements>F:image_contrast_and_color</requirements>
        private void trackBarContrast_Scroll(object sender, System.EventArgs e)
        {
            try
            {
                // Reset the values of parameter 2 and 3
                plugInInterface.Param2 = 0;
                plugInInterface.Param3 = 0;
                // Set the name of the operation to be applied
                plugInInterface.SliderName = "Contrast";
                plugInInterface.Param1 = trackBarContrast.Value;
                // Creates a copy of the thumbnail image to be displyed in
                // the preview pane, showing the applied changes.
                picturePreView.Image = (Image)thumbImage.Clone();
                plugInInterface.TmpBitmap = (Bitmap)picturePreView.Image;
                
                // Call to apply the requestd operation to
                // the thumbnail image.
                applyImageProcessing();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Applies the current image processing
        /// operation to the currently loade image.
        /// </summary>
        private void ApplyProcessingButton_Click(object sender, System.EventArgs e)
        {
            // Store a copy of teh loaded image on which the 
            // image processing calculation is performed.
            plugInInterface.TmpBitmap = (Bitmap)PictureObject.LoadedImage;
            // Use a threadppol and add a work item to the list of items that need
            // to be calculated.
            plugInInterface.ApplyImageProcessing();
            // Let's see the result
            DisplayImage();
            
        }

        /// <summary>
        /// Resets the image to the originally
        /// loaded image.
        /// </summary>
        private void IPResetButton_Click(object sender, 
            System.EventArgs e)
        {
            this.ResetButton_Click(sender, e);
            // Create a new thumbImage and display it in the Preview pane.
            thumbImage = PictureObject.LoadedImage.GetThumbnailImage(
                picturePreView.Width, 
                picturePreView.Height,null, IntPtr.Zero);
            picturePreView.Image = (Image)thumbImage.Clone();
        }

        /// <summary>
        /// Event handler for the scroll event of 
        /// the brightness control. Calls 
        /// applyImageProcessing()
        /// </summary>
        /// <param name="sender">Sender object.</param>
        /// <param name="e">Event arguments.</param>
        /// <requirements>F:image_brightness</requirements>
        private void trackBarBrightness_Scroll(object sender, System.EventArgs e)
        {
            try
            {
                // reset tool
                toolSelected = toolSelection.None;
                // Name of operation.
                plugInInterface.SliderName = "Brightness";
                // Set sliderValue to the value of the 
                // trackBar.
                plugInInterface.Param1 = trackBarBrightness.Value;
                // Reset other sliders
                plugInInterface.Param3 = plugInInterface.Param2 = 0;
                // Copy thumbImage and display it.
                plugInInterface.TmpBitmap = (Bitmap)thumbImage.Clone();
                picturePreView.Image = plugInInterface.TmpBitmap;
                // Call the Plug-In via applyImageProcess.
                applyImageProcessing();
				
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Event handler for the scroll event of 
        /// the color control. Calls 
        /// applyImageProcessing(...)
        /// </summary>
        /// <param name="sender">Sender object.</param>
        /// <param name="e">Event arguments.</param>
        /// <requirements>F:image_contrast_and_color</requirements>
        private void trackBarRed_Scroll(object sender, System.EventArgs e)
        {
            try
            {
                // reset tool
                toolSelected = toolSelection.None;
                // Set operation.
                plugInInterface.SliderName = "Color";
                // Set trackBar value.
                plugInInterface.Param1 = trackBarRed.Value;
                // Apply operation to thumbImage
                applyImageProcessing();
				
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Event handler for the scroll event of 
        /// the color control. Calls 
        /// applyImageProcessing(...)
        /// </summary>
        /// <param name="sender">Sender object.</param>
        /// <param name="e">Event arguments.</param>
        /// <requirements>F:image_contrast_and_color</requirements>
        private void trackBarGreen_Scroll(object sender, System.EventArgs e)
        {
            try
            {
                // reset tool
                toolSelected = toolSelection.None;
                // Set operation
                plugInInterface.SliderName = "Color";
                // Set trackBar value.
                plugInInterface.Param2 = trackBarGreen.Value;
                // apply operation to thumbImage.
                applyImageProcessing();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
        }

        /// <summary>
        /// Event handler for the scroll event of 
        /// the color control. Calls 
        /// applyImageProcessing(...)
        /// </summary>
        /// <param name="sender">Sender object.</param>
        /// <param name="e">Event arguments.</param>
        /// <requirements>F:image_contrast_and_color</requirements>
        private void trackBarBlue_Scroll(object sender, System.EventArgs e)
        {
            try
            {
                // reset tool
                toolSelected = toolSelection.None;
                // Set operation.
                plugInInterface.SliderName = "Color";
                // Set trackBar value.
                plugInInterface.Param3 = trackBarBlue.Value;
                // Apply operation to thumbImage.
                applyImageProcessing();
            }
            catch(Exception exception)
            {
                ExceptionManager.Publish(exception);
            }
		
        }

        /// <summary>
        /// Sets the flag for redEyeTool to true.
        /// If Mouse is clicked then the apply
        /// method is called.
        /// </summary>
        /// <param name="sender">Sender object.</param>
        /// <param name="e">Event arguments.</param>
        /// <requirements>F:image_special_effects</requirements>
        private void redEyeButton_Click(object sender, System.EventArgs e)
        {
            toolSelected = (toolSelected == toolSelection.RedEyeTool ? toolSelection.None : toolSelection.RedEyeTool);
        
        }

		/// <summary>
		/// Invokes the font selection dialog for 3D text.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void fontButton_Click(object sender, System.EventArgs e)
		{
			fontDialog.ShowDialog();
		}

		/// <summary>
		/// Invokes teh color selection dialog for 3D text.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void colorButton_Click(object sender, System.EventArgs e)
		{
			colorDialog.ShowDialog();
		}

		/// <summary>
		/// Event handler that renders 3D text into the image.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void textApplyButton_Click(object sender, System.EventArgs e)
		{
			// Apply was pressed, copy all attributes to OGL3DText
			OGL3DText.Image = this.PictureObject.LoadedImage;
			OGL3DText.Color = colorDialog.Color;
			OGL3DText.Text = this.textBox.Text;
			OGL3DText.Font = fontDialog.Font;

			if (regionComponent != null)
			{
				// use selected region
				OGL3DText.Placement = new Rectangle(
					regionComponent.StartPoint.X,
					regionComponent.StartPoint.Y,
					regionComponent.EndPoint.X - regionComponent.StartPoint.X,
					regionComponent.EndPoint.Y - regionComponent.StartPoint.Y);
			}
			else
			{
				// full image
				OGL3DText.Placement = new Rectangle(
					0, 0,
					this.PictureObject.LoadedImage.Width,
					this.PictureObject.LoadedImage.Height);
			}
			OGL3DText.RotationX = System.Convert.ToInt32(rotationX.Value);
			OGL3DText.RotationY = System.Convert.ToInt32(rotationY.Value);
			OGL3DText.RotationZ = System.Convert.ToInt32(rotationZ.Value);

			// render text and replace LoadedImage
			this.PictureObject.LoadedImage = OGL3DText.Render();

			// now update display
			this.DisplayImage();
		}

		/// <summary>
		/// Event handler for 3D text undo button.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void textUndoButton_Click(object sender, System.EventArgs e)
		{
			// revert to previous image and update display
			this.PictureObject.Reset();
			this.DisplayImage();
		}

		/// <summary>
		/// Enters the 3D text placement mode.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void placementButton_Click(object sender, System.EventArgs e)
		{
			if(toolSelected == toolSelection.None)
			{
				toolSelected = toolSelection.ROITool;
			}
		}
        
    }
}
