using System;
using System.Drawing;
using NUnit.Framework;
using System.Collections;

namespace Photo_Editor_Application
{
	/// <summary>
	/// Summary description for UnitTest.
	/// </summary>
	[TestFixture]
	public class UnitTest
	{
        /// <summary>
        /// Constructor for unit test class.
        /// </summary>
		public UnitTest()
		{
			//
			// TODO: Add constructor logic here
			//
		}
        /// <summary>
        /// Test set up method.
        /// </summary>
        [SetUp] public void Init() 
        {	
            Console.WriteLine("********* New Test-case: *******");
            Console.WriteLine("photo_editor");
            TestImage = new Photo_Editor_Application.Picture();
        }
		
        private Photo_Editor_Application.Picture TestImage;
        private bool amIPassed = false;
        /// <summary>
        /// Tear down method for test case.
        /// </summary>
        [TearDown] public void Destroy()
        {
            TestImage.LoadedImage.Dispose();
            if(amIPassed)
            {	
                Console.WriteLine("=> PASSED");
            }
            else
            {
                Console.WriteLine("%%%%%% Failed");	
            }
        }
        /// <summary>
        /// Test for F:image_load_and_save.
        /// Windows standrad dialogs are used so really
        /// tested is only if the default image loaded has the 
        /// dimensions
        /// that are expected!
        /// </summary>
        /// <requirement>F:image_load_and_save</requirement>
        [Test] public void LoadandSaveImageTest() 
        {
            const string fileName = "Test.jpg";	
			
            Console.WriteLine("image_load_and_save");
			
            Assertion.AssertEquals("Load Image, width", 
                TestImage.LoadedImage.Width, 2048);
            Assertion.AssertEquals("Load Image, height", 
                TestImage.LoadedImage.Height, 1536);
			
            TestImage.CropImage(200, 400);
            TestImage.SaveImage(fileName);

            TestImage.LoadImage("Hawaii.jpg");

            TestImage.LoadImage(fileName);

            Assertion.AssertEquals("Load Image, width", 
                TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Load Image, height", 
                TestImage.LoadedImage.Height, 400);
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_crop.
        /// Crops the default image and sees if dimensions are 
        /// changing accordingly!
        /// </summary>
        /// <requirement>F:image_crop</requirement>
        [Test] public void ImageCropTest()
        {
            Console.WriteLine("image_crop");

            TestImage.CropImage(200, 400);
            Assertion.AssertEquals("Crop Image, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Crop Image, height", TestImage.LoadedImage.Height, 400);
            TestImage.CropImage(125, 238);
            Assertion.AssertEquals("Crop Image, width", TestImage.LoadedImage.Width, 125);
            Assertion.AssertEquals("Crop Image, height", TestImage.LoadedImage.Height, 238);

            TestImage.CropImage(200, 400);
            Assertion.AssertEquals("Crop Image, width", TestImage.LoadedImage.Width, 125);
            Assertion.AssertEquals("Crop Image, height", TestImage.LoadedImage.Height, 238);

            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_rotate.
        /// Tests different rotations by
        /// verifying the dimensions after rotation.
        /// </summary>
        /// <requirement>F:image_rotate</requirement>
        [Test] public void ImageRotateTest()
        {
            Console.WriteLine("image_rotate");
			
            TestImage.CropImage(200, 400);
            TestImage.RotateFlipImage(RotateFlipType.Rotate90FlipNone);
			
            Assertion.AssertEquals("Rotate Image 1, width", TestImage.LoadedImage.Width, 400);
            Assertion.AssertEquals("Rotate Image 1, height", TestImage.LoadedImage.Height, 200);

            TestImage.RotateFlipImage(RotateFlipType.Rotate90FlipNone);

            Assertion.AssertEquals("Rotate Image 2, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Rotate Image 2, height", TestImage.LoadedImage.Height, 400);

            TestImage.RotateFlipImage(RotateFlipType.Rotate180FlipNone);

            Assertion.AssertEquals("Rotate Image 3, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Rotate Image 3, height", TestImage.LoadedImage.Height, 400);

            amIPassed = true;
        }
		
        /// <summary>
        /// Test for F:image_flip.
        /// Flips the image and checks if the dimensions 
        /// are changing accordingly.
        /// </summary>
        /// <requirement>F:image_flip</requirement>
        [Test] public void ImageFlipTest()
        {
            Console.WriteLine("image_flip");
			
            TestImage.CropImage(200, 400);
            TestImage.RotateFlipImage(RotateFlipType.RotateNoneFlipX);
			
            Assertion.AssertEquals("Flip Image 1, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Flip Image 1, height", TestImage.LoadedImage.Height, 400);

            TestImage.RotateFlipImage(RotateFlipType.RotateNoneFlipY);

            Assertion.AssertEquals("Flip Image 2, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Flip Image 2, height", TestImage.LoadedImage.Height, 400);

            TestImage.RotateFlipImage(RotateFlipType.RotateNoneFlipXY);

            Assertion.AssertEquals("Flip Image 3, width", TestImage.LoadedImage.Width, 200);
            Assertion.AssertEquals("Flip Image 3, height", TestImage.LoadedImage.Height, 400);

            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_annotations.
        /// Sets the start and end point and 
        /// checks they were set correctly.
        /// Also checks if picking is done
        /// correctly.
        /// </summary>
        /// <requirement>F:image_graphics_annotations</requirement>
        [Test] public void LineTest()
        {
            Console.WriteLine("image_graphics_annotations");
            LineComponent myLine = new LineComponent();
            myLine.StartPoint = new Point(0, 0);
            myLine.EndPoint = new Point(100, 200);
            Assertion.AssertEquals("Line (0, 0) (100, 200)", myLine.StartPoint.X, 0);
            Assertion.AssertEquals("Line (0, 0) (100, 200)", myLine.StartPoint.Y, 0);
            Assertion.AssertEquals("Line (0, 0) (100, 200)", myLine.EndPoint.X, 100);
            Assertion.AssertEquals("Line (0, 0) (100, 200)", myLine.EndPoint.Y, 200);
            Assertion.AssertEquals("Pick line test", myLine.IsPicked(0, 0), true);
            Assertion.AssertEquals("Pick line test", myLine.IsPicked(97, 198), true);
            Assertion.AssertEquals("Pick line test", myLine.IsPicked(95, 200), false);
				

            myLine.StartPoint = new Point(200, 10);
            myLine.EndPoint = new Point(10, 5);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myLine.StartPoint.X, 200);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myLine.StartPoint.Y, 10);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myLine.EndPoint.X, 10);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myLine.EndPoint.Y, 5);
            Assertion.AssertEquals("Pick line test", myLine.IsPicked(197, 12), true);
            Assertion.AssertEquals("Pick line test", myLine.IsPicked(13, 7), true);
            Assertion.AssertEquals("Pick line test", myLine.IsPicked(0, 10), false);
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_annotations.
        /// Sets the start and end point and 
        /// checks they were set correctly.
        /// Also checks if picking is done
        /// correctly.
        /// </summary>
        /// <requirement>F:image_graphics_annotations</requirement>
        [Test] public void RectangleTest()
        {
            Console.WriteLine("image_graphics_annotations");
            RectangleComponent myRect = new RectangleComponent();
            myRect.StartPoint = new Point(0, 0);
            myRect.EndPoint = new Point(100, 200);
            Assertion.AssertEquals("Rectangle (0, 0) (100, 200)", myRect.StartPoint.X, 0);
            Assertion.AssertEquals("Rectangle (0, 0) (100, 200)", myRect.StartPoint.Y, 0);
            Assertion.AssertEquals("Rectangle (0, 0) (100, 200)", myRect.EndPoint.X, 100);
            Assertion.AssertEquals("Rectangle (0, 0) (100, 200)", myRect.EndPoint.Y, 200);
            Assertion.AssertEquals("Pick rectangle test", myRect.IsPicked(0, 0), true);
            Assertion.AssertEquals("Pick rectangle test", myRect.IsPicked(97, 198), true);
            Assertion.AssertEquals("Pick rectangle test", myRect.IsPicked(95, 200), false);

            myRect.StartPoint = new Point(200, 10);
            myRect.EndPoint = new Point(10, 5);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myRect.StartPoint.X, 200);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myRect.StartPoint.Y, 10);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myRect.EndPoint.X, 10);
            Assertion.AssertEquals("Rectangle (200, 10) (10, 5)", myRect.EndPoint.Y, 5);
            Assertion.AssertEquals("Pick rectangle test", myRect.IsPicked(197, 12), true);
            Assertion.AssertEquals("Pick rectangle test", myRect.IsPicked(13, 7), true);
            Assertion.AssertEquals("Pick rectangle test", myRect.IsPicked(0, 10), false);
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_annotations.
        /// Sets the start and end point and
        /// checks the coordinates were set correctly. 
        /// Also checks if picking is done
        /// correctly.
        /// </summary>
        /// <requirement>F:image_graphics_annotations</requirement>
        [Test] public void CircleTest()
        {
            Console.WriteLine("image_graphics_annotations");
            CircleComponent myC = new CircleComponent();
            myC.StartPoint = new Point(0, 0);
            myC.EndPoint = new Point(100, 200);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.StartPoint.X, 0);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.StartPoint.Y, 0);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.EndPoint.X, 100);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.EndPoint.Y, 200);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(0, 0), true);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(97, 198), true);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(95, 200), false);

            myC.StartPoint = new Point(205, 10);
            myC.EndPoint = new Point(10, 5);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.StartPoint.X, 205);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.StartPoint.Y, 10);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.EndPoint.X, 10);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.EndPoint.Y, 5);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(12, 2), true);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(97, 198), false);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(201, 7), true);

            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_annotations.
        /// Sets the text, start and end point and checks if they 
        /// were set correctly. Also checks if picking is done
        /// correctly.
        /// </summary>
        /// <requirement>F:image_graphics_annotations</requirement>
        [Test] public void TextTest()
        {
            Console.WriteLine("image_text_annotations");
            TextComponent myT = new TextComponent();
            myT.StartPoint = new Point(100, 50);
            myT.TextInput = "Test";
			
            Assertion.AssertEquals("Text (100, 50) ", myT.StartPoint.X, 100);
            Assertion.AssertEquals("Text (100, 50) ", myT.StartPoint.Y, 50);
            Assertion.AssertEquals("Text: Test", myT.TextInput, "Test");
            Assertion.AssertEquals("Pick text test", myT.IsPicked(98, 51), true);
            Assertion.AssertEquals("Pick text test", myT.IsPicked(103, 47), true);
            Assertion.AssertEquals("Pick text test", myT.IsPicked(97, 200), false);

            myT.StartPoint = new Point(8, 287);
            myT.TextInput = "Text to be checked here";
			
            Assertion.AssertEquals("Text (8, 287) ", myT.StartPoint.X, 8);
            Assertion.AssertEquals("Text (8, 287) ", myT.StartPoint.Y, 287);
            Assertion.AssertEquals("Text: Test", myT.TextInput, "Text to be checked here");
            Assertion.AssertEquals("Pick text test", myT.IsPicked(5, 289), true);
            Assertion.AssertEquals("Pick text test", myT.IsPicked(10, 290), true);
            Assertion.AssertEquals("Pick text test", myT.IsPicked(8, 200), false);
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_annotations.
        /// Creates GraphicsComponents, puts them in the list and 
        /// removes them to check if list is handeled 
        /// correctly.
        /// </summary>
        /// <requirement>F:image_graphics_annotations</requirement>
        /// <requirement>F:image_text_annotations</requirement>
        [Test] public void PictureListTest()
        {
            Console.WriteLine("image_graphics_annotations");
            int i;
            ArrayList lineList = new ArrayList(10);
			
            for(i = 0; i < 10; ++i)
            {
                lineList.Add(new LineComponent());
                TestImage.Add((LineComponent)lineList[i]);
            }
            ArrayList cList = TestImage.GraphicsComponentList;
            for(i = 0; i < 10; ++i)
            {
                Assertion.AssertEquals("Line list test ", lineList[i], cList[i]);
                Assertion.AssertEquals("Line list test ", lineList.Count, 10);
            }

            TestImage.Remove((GraphicsComponent)lineList[3]);
            lineList.Remove(lineList[3]);
            for(i = 0; i < 9; ++i)
            {
                Assertion.AssertEquals("Line list test ", lineList[i], cList[i]);
                Assertion.AssertEquals("Line list test ", lineList.Count, 9);
            }

            TestImage.Remove((GraphicsComponent)lineList[1]);
            lineList.Remove(lineList[1]);
            for(i = 0; i < 8; ++i)
            {
                Assertion.AssertEquals("Line list test ", lineList[i], cList[i]);
                Assertion.AssertEquals("Line list test ", lineList.Count, 8);
            }

            Assertion.AssertEquals("Line list test ", lineList.Count, cList.Count);
			
            lineList.Clear();
            cList.Clear();
            Assertion.AssertEquals("Line list test ", lineList.Count, cList.Count);


            amIPassed = true;
        }

        /// <summary>
        /// Test for F:image_graphics_annotations.
        /// Sets the start point and 
        /// checks the coordinate
        /// transformation.
        /// </summary>
        /// <requirement>F:image_graphics_annotations</requirement>
        [Test] public void CoordTrafoTest()
        {
            Console.WriteLine("image_graphics_annotations");
            RectangleComponent myRect = new RectangleComponent();
            myRect.StartPoint = new Point(0, 0);
			
            GraphicsComponent.ViewportCoords =  new Point(10, 500);


            Assertion.AssertEquals("Rectangle Viewport Coords (0, 0) => (10, 500)", myRect.CalcViewportCoords(myRect.StartPoint).X, -10);
            Assertion.AssertEquals("Rectangle Viewport Coords (0, 0) => (10, 500)", myRect.CalcViewportCoords(myRect.StartPoint).Y, -500);
			
            myRect.StartPoint = new Point(200, 651);
			
            Assertion.AssertEquals("Rectangle Viewport Coords (200, 651) => (10, 500)", myRect.CalcViewportCoords(myRect.StartPoint).X, 190);
            Assertion.AssertEquals("Rectangle Viewport Coords (200, 651) => (10, 500)", myRect.CalcViewportCoords(myRect.StartPoint).Y, 151);
			
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_special_effects.
        /// Sets the start and end point and 
        /// checks they were set correctly.
        /// Also checks if picking is done
        /// correctly and if the static pen in the
        /// GraphicsComponent is set correctly.
        /// </summary>
        /// <requirement>F:image_graphics_special_effects</requirement>
        [Test] public void RectFrameTest()
        {
            Pen testPen = new Pen(Color.Red, 7);
            GraphicsComponent.DrawingPen = testPen;
            Console.WriteLine("image_graphics_special_effects");
            RectFrameComponent myRect = new RectFrameComponent();
            myRect.StartPoint = new Point(0, 0);
            myRect.EndPoint = new Point(100, 200);
            Assertion.AssertEquals("Rectangle Frame (0, 0) (100, 200)", myRect.StartPoint.X, 0);
            Assertion.AssertEquals("Rectangle Frame (0, 0) (100, 200)", myRect.StartPoint.Y, 0);
            Assertion.AssertEquals("Rectangle Frame (0, 0) (100, 200)", myRect.EndPoint.X, 100);
            Assertion.AssertEquals("Rectangle Frame (0, 0) (100, 200)", myRect.EndPoint.Y, 200);
            Assertion.AssertEquals("Pick Rectangle Frame test", myRect.IsPicked(0, 0), true);
            Assertion.AssertEquals("Pick Rectangle Frame test", myRect.IsPicked(97, 198), true);
            Assertion.AssertEquals("Pick Rectangle Frame test", myRect.IsPicked(95, 200), false);

            myRect.StartPoint = new Point(200, 10);
            myRect.EndPoint = new Point(10, 5);
            Assertion.AssertEquals("Rectangle Frame (200, 10) (10, 5)", myRect.StartPoint.X, 200);
            Assertion.AssertEquals("Rectangle Frame (200, 10) (10, 5)", myRect.StartPoint.Y, 10);
            Assertion.AssertEquals("Rectangle Frame (200, 10) (10, 5)", myRect.EndPoint.X, 10);
            Assertion.AssertEquals("Rectangle Frame (200, 10) (10, 5)", myRect.EndPoint.Y, 5);
            Assertion.AssertEquals("Pick Rectangle Frame test", myRect.IsPicked(197, 12), true);
            Assertion.AssertEquals("Pick Rectangle Frame test", myRect.IsPicked(13, 7), true);
            Assertion.AssertEquals("Pick Rectangle Frame test", myRect.IsPicked(0, 10), false);

            Assertion.AssertEquals("Pen Rectangle Frame color test", GraphicsComponent.DrawingPen.Color, Color.Red);
            Assertion.AssertEquals("Pen Rectangle Frame width test", GraphicsComponent.DrawingPen.Width, 7);
            testPen.Dispose();
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_graphics_special_effects.
        /// Sets the start and end point and
        /// checks the coordinates were set correctly. 
        /// Also checks if picking is done
        /// correctly and if the static pen in the
        /// GraphicsComponent is set correctly.
        /// </summary>
        /// <requirement>F:image_graphics_special_effects</requirement>
        [Test] public void EllipseFrameTest()
        {
            Console.WriteLine("image_graphics_special_effects");
            Pen testPen = new Pen(Color.WhiteSmoke, 2);
            GraphicsComponent.DrawingPen = testPen;
            EllipseFrameComponent myC = new EllipseFrameComponent();
            myC.StartPoint = new Point(0, 0);
            myC.EndPoint = new Point(100, 200);
			
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.StartPoint.X, 0);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.StartPoint.Y, 0);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.EndPoint.X, 100);
            Assertion.AssertEquals("Circle (0, 0) (100, 200)", myC.EndPoint.Y, 200);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(0, 0), true);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(97, 198), true);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(95, 200), false);

            myC.StartPoint = new Point(205, 10);
            myC.EndPoint = new Point(10, 5);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.StartPoint.X, 205);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.StartPoint.Y, 10);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.EndPoint.X, 10);
            Assertion.AssertEquals("Rectangle (205, 10) (10, 5)", myC.EndPoint.Y, 5);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(12, 2), true);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(97, 198), false);
            Assertion.AssertEquals("Pick circle test", myC.IsPicked(201, 7), true);
            
            Assertion.AssertEquals("Pen Rectangle Frame test", GraphicsComponent.DrawingPen.Color, Color.WhiteSmoke);
            Assertion.AssertEquals("Pen Rectangle Frame width test", GraphicsComponent.DrawingPen.Width, 2);
            testPen.Dispose();
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_contrast_and_color.
        /// Checks if the contrast plug in
        /// can be loaded at runtime and if the
        /// calculaion is correctly performed on the image.
        /// </summary>
        /// <requirement>F:image_contrast_and_color</requirement>
        [Test] public void ContrastPlugInTest()
        {
            const int width = 100;
            const int height = 75;
  
            Console.WriteLine("image_contrast");
            // Create a new bitmap.
            Bitmap testImage = new Bitmap(width, height);

            PlugInInterface loader = new PlugInInterface();
            // Set all pixels in the image to black,
            // except of the center pixel which is set to
            // Red = 100, Green = 75, Blue = 50)
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    if( i == ((height/2)) && (j == (width/2)))
                    {
                        testImage.SetPixel(j, i, Color.FromArgb(100, 75, 50));        
                    }
                    else 
                    {
                        testImage.SetPixel(j, i, Color.FromArgb(0, 0, 0));
                    }
                }
            }
            loader.TmpBitmap = testImage;
            loader.SliderName = "Contrast";
            loader.Param1 = 1;
            loader.Param2 = loader.Param3 = 0;
            // Invoke the ProcessImage method from the
            // Contrast PlugIn.
            loader.ApplyImageProcessing();
            // Test the center Pixel color value for correctness
            Color pixColor = testImage.GetPixel(width/2, height/2);
            Assertion.AssertEquals("Center Pixel Red Value", 
                pixColor.R, 255);
            Assertion.AssertEquals("Center Pixel Green Value", 
                pixColor.G, 243);
            Assertion.AssertEquals("Center Pixel Value", pixColor.B, 162);
      
            // Check if position on pixel before the centerPixel is 0 value 
            // color.
            pixColor = testImage.GetPixel(width/2 -1, height/2);
            Assertion.AssertEquals("Center Pixel Red Value", 
                pixColor.R, 0);
            Assertion.AssertEquals("Center Pixel Green Value", 
                pixColor.G, 0);
            Assertion.AssertEquals("Center Pixel Value", pixColor.B, 0);
      
            testImage.Dispose();
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_contrast_and_color.
        /// Checks if the color plug in
        /// can be loaded at runtime and if the
        /// calculaion is correctly performed on the image.
        /// </summary>
        /// <requirement>F:image_contrast_and_color</requirement>
        [Test] public void ColorPlugInTest()
        {
            const int width = 125;
            const int height = 90;
            Color pixColor;
            PlugInInterface loader = new PlugInInterface();
            Console.WriteLine("image_contrast_and_color");
            // Create a new bitmap.
            Bitmap testImage = new Bitmap(width, height);
            // Set all pixels in the image to, 
            // Red = 110, Green = 99, Blue = 76)
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    testImage.SetPixel(j, i, Color.FromArgb(110, 99, 76));
                }
            }
            // Invoke the ProcessImage method from the
            // Color PlugIn.
            loader.TmpBitmap = testImage;
            loader.SliderName = "Color";
            loader.Param1 = 55;
            loader.Param2 = 8;
            loader.Param3 = 100;
            loader.ApplyImageProcessing();
            // Wlak through the entire image
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    // Check each pixel and colro component
                    // for correctness.
                    pixColor = testImage.GetPixel(j, i);
                    Assertion.AssertEquals("Pixel Red Value", 
                        pixColor.R, 165);
                    Assertion.AssertEquals("Pixel Green Value", 
                        pixColor.G, 107);
                    Assertion.AssertEquals("Pixel Value", pixColor.B, 176);
                }
            }
            testImage.Dispose();
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_special_efects.
        /// Checks if the red eye plug in
        /// can be loaded at runtime and if the
        /// calculation is correctly performed on the image.
        /// </summary>
        /// <requirement>F:image_special_effects</requirement>
        [Test] public void RedEyePlugInTest()
        {
            const int width = 75;
            const int height = 90;
            Color pixColor;
            PlugInInterface loader = new PlugInInterface();
            Console.WriteLine("image_contrast_and_color");
            // Create a new bitmap.
            Bitmap testImage = new Bitmap(width, height);
            // Set all pixels in the image to, 
            // Red = 100, Green = 198, Blue = 55)
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    testImage.SetPixel(j, i, Color.FromArgb(100, 198, 55));
                }
            }
            // Invoke the ProcessImage method from the
            // Color PlugIn.
            loader.TmpBitmap = testImage;
            loader.SliderName = "RedEye";
            loader.Param1 = 50;
            loader.Param2 = 45;
            loader.Param3 = 10;
            loader.ApplyImageProcessing();
            // Walk through the entire image
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    // Check each pixel and color component
                    // for correctness.
                    pixColor = testImage.GetPixel(j, i);
                    if(((j  >= 50) && (j < 60)) && ((i  >= 45) && (i < 55)))
                    {
                        Assertion.AssertEquals("Pixel Red Value", 
                            pixColor.R, 80);
                    }
                    else
                    {
                        Assertion.AssertEquals("Pixel Red Value", 
                            pixColor.R, 100);
                    }
                    Assertion.AssertEquals("Pixel Green Value", 
                        pixColor.G, 198);
                    Assertion.AssertEquals("Pixel Value", pixColor.B, 55);
                }
            }
  
            testImage.Dispose();
            amIPassed = true;
        }
        /// <summary>
        /// Test for F:image_brightness.
        /// Checks if the brightness plug in
        /// can be loaded at runtime and if the
        /// calculaion is correctly performed on the image.
        /// </summary>
        /// <requirement>F:image_brightness</requirement>
        [Test] public void BrightnessPlugInTest()
        {
            const int width = 75;
            const int height = 90;
            Color pixColor;
            PlugInInterface loader = new PlugInInterface();
            Console.WriteLine("image_brightness");
            // Create a new bitmap.
            Bitmap testImage = new Bitmap(width, height);
            // Set all pixels in the image to, 
            // Red = 100, Green = 198, Blue = 55)
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    testImage.SetPixel(j, i, Color.FromArgb(100, 198, 55));
                }
            }
            // Invoke the ProcessImage method from the
            // Color PlugIn.
            loader.TmpBitmap = testImage;
            loader.SliderName = "Brightness";
            loader.Param1 = 10;
            loader.Param2 = 0;
            loader.Param3 = 0;
            loader.ApplyImageProcessing();
            // Wlak through the entire image
            for(int i = 0; i < height; ++i)
            {
                for(int j = 0; j < width; ++j)
                {
                    // Check each pixel and colro component
                    // for correctness.
                    pixColor = testImage.GetPixel(j, i);
                    Assertion.AssertEquals("Pixel Red Value", pixColor.R, 106);
                    Assertion.AssertEquals("Pixel Green Value", pixColor.G, 210);
                    Assertion.AssertEquals("Pixel Value", pixColor.B, 58);
                }
            }
			
            testImage.Dispose();
            amIPassed = true;
        }

		/// <summary>
		/// Compares two bitmaps.
		/// </summary>
		/// <param name="bitmap1">First bitmap.</param>
		/// <param name="bitmap2">Second bitmap.</param>
		/// <returns>True if the two bitmaps are identical, otherwise false.</returns>
		public bool CompareBitmaps(Bitmap bitmap1, Bitmap bitmap2)
		{
			// compare image size
			if (bitmap1.Size != bitmap2.Size)
			{
				return false;
			}

			// compare all pixels
			for (int y = 0; y < bitmap1.Height; y++)
			{
				for (int x = 0; x < bitmap1.Width; x++)
				{
					if (bitmap1.GetPixel(x, y) != bitmap2.GetPixel(x, y))
						return false;
				}
			}

			// size and all pixels are the same
			return true;
		}

		/// <summary>
		/// Renders 3 strings in the different colors and orientation.
		/// </summary>
		/// <requirement>F:image_3dtext</requirement>
		/// <requirement>F:image_3dtext</requirement>
		/// <requirement>F:image_3dtext_color</requirement>
		/// <requirement>F:image_3dtext_font</requirement>
		/// <requirement>F:image_3dtext_rotate</requirement>
		/// <requirement>C:image_3dtext_singleline</requirement>
		[Test]
		public void OGL3DTextTest() 
		{
			Bitmap output = null;

			Bitmap input = new Bitmap(800, 600);
			// Set each pixel in image to white.
			for (int Xcount = 0; Xcount < input.Width; Xcount++)
			{
				for (int Ycount = 0; Ycount < input.Height; Ycount++)
				{
					input.SetPixel(Xcount, Ycount, Color.DarkGray);
				}
			}

			OGL3DText.Image = input;
			OGL3DText.Color = Color.Red;
			OGL3DText.Text = "OpenGL";
			OGL3DText.Font = new Font("Arial Black", 40);
			OGL3DText.Placement = new Rectangle(0, 0, 400, 400);
			OGL3DText.RotationX = 30;
			output = OGL3DText.Render();
			Assertion.AssertNotNull("First string.", output);

			OGL3DText.Image = output;
			OGL3DText.Color = Color.LightBlue;
			OGL3DText.Text = "and";
			OGL3DText.Placement = new Rectangle(200, 0, 600, 600);
			OGL3DText.RotationX = 0;
			OGL3DText.RotationY = 60;
			output = OGL3DText.Render();
			Assertion.AssertNotNull("Second string.", output);

			OGL3DText.Image = output;
			OGL3DText.Color = Color.LightGreen;
			OGL3DText.Text = ".NET";
			OGL3DText.Placement = new Rectangle(400, 200, 400, 400);
			OGL3DText.RotationY = 0;
			OGL3DText.RotationZ = 30;
			output = OGL3DText.Render();
			Assertion.AssertNotNull("Third string.", output);

			/*/ temporarily remove * on the left to create the reference output
			if (output != null)
			{
				output.Save("output.png", System.Drawing.Imaging.ImageFormat.Png);
			}
			// */

			// load reference image and compare with the one just rendered
			Bitmap reference = new Bitmap("output.png");
			Assertion.AssertEquals("Compare with reference image", true, CompareBitmaps(output, reference));
		}

    }
}


